precision highp float;

uniform float time;
uniform float powerFactor;
uniform float multiplicator;
uniform vec3 lightPosition;
uniform float specularIntensity;
uniform float shininess;

// Assuming injected by Three.js:
// uniform vec3 cameraPosition;

varying vec3 vPosition;
varying vec3 vNormal;

vec3 hsl2rgb(float h, float s, float l) {
    float r, g, b;
    if (s == 0.0) {
        r = g = b = l; // achromatic
    } else {
        float q = l < 0.5 ? l * (1.0 + s) : l + s - l * s;
        float p = 2.0 * l - q;
        float hk = h / 360.0; // converting to range [0,1] for calculation
        float t[3];
        t[0] = hk + 1.0/3.0;
        t[1] = hk;
        t[2] = hk - 1.0/3.0;
        for(int i = 0; i < 3; i++) {
            if (t[i] < 0.0) t[i] += 1.0;
            if (t[i] > 1.0) t[i] -= 1.0;
            if (t[i] < 1.0/6.0)
                t[i] = p + (q - p) * 6.0 * t[i];
            else if (t[i] < 0.5)
                t[i] = q;
            else if (t[i] < 2.0/3.0)
                t[i] = p + (q - p) * (2.0/3.0 - t[i]) * 6.0;
            else
                t[i] = p;
        }
        r = t[0];
        g = t[1];
        b = t[2];
    }
    return vec3(r, g, b);
}

void main() {
    vec3 ambient = 0.1 * vec3(1.0, 1.0, 1.0);
    vec3 lightDir = normalize(lightPosition - vPosition);
    vec3 viewDir = normalize(-vPosition); // Assuming camera at the origin
    vec3 reflectDir = reflect(-lightDir, vNormal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), shininess);
    vec3 specular = specularIntensity * spec * vec3(1.0, 1.0, 1.0);
    float diff = max(dot(vNormal, lightDir), 0.0);
    vec3 diffuse = diff * vec3(1.0, 1.0, 1.0);
    vec3 cameraDirection = normalize(cameraPosition - vPosition);
    float fresnel = dot(vNormal, cameraDirection);
    float iridescence = pow(fresnel, powerFactor) * multiplicator;
    float f = mod(iridescence + time, 1.0);
    vec3 iridescentColor = hsl2rgb(f * 360.0, 0.7, 0.5);
    vec3 finalColor = ambient + diffuse + specular + iridescentColor;
    gl_FragColor = vec4(finalColor, 1.0);
}
